@@ -39,47 +39,58 @@ _cargo()
 	local opt_common="$opt_help $opt_verbose $opt_quiet $opt_color"
 	local opt_pkg_spec='-p --package --all --exclude --workspace'
 	local opt_pkg='-p --package'
-	local opt_feat='--features --all-features --no-default-features'
+	local opt_feat='-F --features --all-features --no-default-features'
 	local opt_mani='--manifest-path'
 	local opt_jobs='-j --jobs'
+	local opt_parallel="$opt_jobs --keep-going"
 	local opt_force='-f --force'
 	local opt_sync='-s --sync'
 	local opt_lock='--frozen --locked --offline'
 	local opt_targets="--lib --bin --bins --example --examples --test --tests --bench --benches --all-targets"
 
 	local opt___nocmd="$opt_common -V --version --list --explain"
-	local opt__bench="$opt_common $opt_pkg_spec $opt_feat $opt_mani $opt_lock $opt_jobs $opt_targets --message-format --target --no-run --no-fail-fast --target-dir"
-	local opt__build="$opt_common $opt_pkg_spec $opt_feat $opt_mani $opt_lock $opt_jobs $opt_targets --message-format --target --release --profile --target-dir"
-	local opt__check="$opt_common $opt_pkg_spec $opt_feat $opt_mani $opt_lock $opt_jobs $opt_targets --message-format --target --release --profile --target-dir"
+	local opt__add="$opt_common -p --package --features --default-features --no-default-features $opt_mani --optional --no-optional --rename --dry-run --path --git --branch --tag --rev --registry --dev --build --target --ignore-rust-version"
+	local opt__bench="$opt_common $opt_pkg_spec $opt_feat $opt_mani $opt_lock $opt_jobs $opt_targets --message-format --target --no-run --no-fail-fast --target-dir --ignore-rust-version"
+	local opt__build="$opt_common $opt_pkg_spec $opt_feat $opt_mani $opt_lock $opt_parallel $opt_targets --message-format --target --release --profile --target-dir --ignore-rust-version"
+	local opt__b="$opt__build"
+	local opt__check="$opt_common $opt_pkg_spec $opt_feat $opt_mani $opt_lock $opt_parallel $opt_targets --message-format --target --release --profile --target-dir --ignore-rust-version"
+	local opt__c="$opt__check"
 	local opt__clean="$opt_common $opt_pkg $opt_mani $opt_lock --target --release --doc --target-dir --profile"
-	local opt__doc="$opt_common $opt_pkg_spec $opt_feat $opt_mani $opt_lock $opt_jobs --message-format --bin --bins --lib --target --open --no-deps --release --document-private-items --target-dir --profile"
+	local opt__clippy="$opt_common $opt_pkg_spec $opt_feat $opt_mani $opt_lock $opt_parallel $opt_targets --message-format --target --release --profile --target-dir --no-deps --fix"
+	local opt__doc="$opt_common $opt_pkg_spec $opt_feat $opt_mani $opt_lock $opt_parallel --message-format --bin --bins --lib --target --open --no-deps --release --document-private-items --target-dir --profile --ignore-rust-version"
+	local opt__d="$opt__doc"
 	local opt__fetch="$opt_common $opt_mani $opt_lock --target"
-	local opt__fix="$opt_common $opt_pkg_spec $opt_feat $opt_mani $opt_jobs $opt_targets $opt_lock --release --target --message-format --broken-code --edition --edition-idioms --allow-no-vcs --allow-dirty --allow-staged --profile --target-dir"
+	local opt__fix="$opt_common $opt_pkg_spec $opt_feat $opt_mani $opt_parallel $opt_targets $opt_lock --release --target --message-format --broken-code --edition --edition-idioms --allow-no-vcs --allow-dirty --allow-staged --profile --target-dir --ignore-rust-version"
 	local opt__generate_lockfile="$opt_common $opt_mani $opt_lock"
 	local opt__help="$opt_help"
+	local opt__info="$opt_common $opt_lock --registry --index"
 	local opt__init="$opt_common $opt_lock --bin --lib --name --vcs --edition --registry"
-	local opt__install="$opt_common $opt_feat $opt_jobs $opt_lock $opt_force --bin --bins --branch --debug --example --examples --git --list --path --rev --root --tag --version --registry --target --profile --no-track"
+	local opt__install="$opt_common $opt_feat $opt_parallel $opt_lock $opt_force --bin --bins --branch --debug --example --examples --git --list --path --rev --root --tag --version --registry --target --profile --no-track --ignore-rust-version"
 	local opt__locate_project="$opt_common $opt_mani $opt_lock --message-format --workspace"
 	local opt__login="$opt_common $opt_lock --registry"
 	local opt__metadata="$opt_common $opt_feat $opt_mani $opt_lock --format-version=1 --no-deps --filter-platform"
 	local opt__new="$opt_common $opt_lock --vcs --bin --lib --name --edition --registry"
 	local opt__owner="$opt_common $opt_lock -a --add -r --remove -l --list --index --token --registry"
-	local opt__package="$opt_common $opt_mani $opt_feat $opt_lock $opt_jobs --allow-dirty -l --list --no-verify --no-metadata --target --target-dir"
+	local opt__package="$opt_common $opt_mani $opt_feat $opt_lock $opt_parallel --allow-dirty -l --list --no-verify --no-metadata --index --registry --target --target-dir"
 	local opt__pkgid="$opt_common $opt_mani $opt_lock $opt_pkg"
-	local opt__publish="$opt_common $opt_mani $opt_feat $opt_lock $opt_jobs --allow-dirty --dry-run --token --no-verify --index --registry --target --target-dir"
-	local opt__read_manifest="$opt_help $opt_quiet $opt_verbose $opt_mani $opt_color $opt_lock --no-deps"
-	local opt__run="$opt_common $opt_pkg $opt_feat $opt_mani $opt_lock $opt_jobs --message-format --target --bin --example --release --target-dir --profile"
-	local opt__rustc="$opt_common $opt_pkg $opt_feat $opt_mani $opt_lock $opt_jobs $opt_targets -L --crate-type --extern --message-format --profile --target --release --target-dir"
-	local opt__rustdoc="$opt_common $opt_pkg $opt_feat $opt_mani $opt_lock $opt_jobs $opt_targets --message-format --target --release --open --target-dir --profile"
+	local opt__publish="$opt_common $opt_mani $opt_feat $opt_lock $opt_parallel --allow-dirty --dry-run --token --no-verify --index --registry --target --target-dir"
+	local opt__remove="$opt_common $opt_pkg $opt_lock $opt_mani --dry-run --dev --build --target"
+	local opt__rm="$opt__remove"
+	local opt__report="$opt_help $opt_verbose $opt_color future-incompat future-incompatibilities"
+	local opt__report__future_incompat="$opt_help $opt_verbose $opt_color $opt_pkg --id"
+	local opt__run="$opt_common $opt_pkg $opt_feat $opt_mani $opt_lock $opt_parallel --message-format --target --bin --example --release --target-dir --profile --ignore-rust-version"
+	local opt__r="$opt__run"
+	local opt__rustc="$opt_common $opt_pkg $opt_feat $opt_mani $opt_lock $opt_parallel $opt_targets -L --crate-type --extern --message-format --profile --target --release --target-dir --ignore-rust-version"
+	local opt__rustdoc="$opt_common $opt_pkg $opt_feat $opt_mani $opt_lock $opt_parallel $opt_targets --message-format --target --release --open --target-dir --profile --ignore-rust-version"
 	local opt__search="$opt_common $opt_lock --limit --index --registry"
-	local opt__test="$opt_common $opt_pkg_spec $opt_feat $opt_mani $opt_lock $opt_jobs $opt_targets --message-format --doc --target --no-run --release --no-fail-fast --target-dir --profile"
+	local opt__test="$opt_common $opt_pkg_spec $opt_feat $opt_mani $opt_lock $opt_jobs $opt_targets --message-format --doc --target --no-run --release --no-fail-fast --target-dir --profile --ignore-rust-version"
+	local opt__t="$opt__test"
 	local opt__tree="$opt_common $opt_pkg_spec $opt_feat $opt_mani $opt_lock --target -i --invert --prefix --no-dedupe --duplicates -d --charset -f --format -e --edges"
 	local opt__uninstall="$opt_common $opt_lock $opt_pkg --bin --root"
-	local opt__update="$opt_common $opt_mani $opt_lock $opt_pkg --aggressive --precise --dry-run"
+	local opt__update="$opt_common $opt_mani $opt_lock $opt_pkg --aggressive --recursive --precise --dry-run"
 	local opt__vendor="$opt_common $opt_mani $opt_lock $opt_sync --no-delete --respect-source-config --versioned-dirs"
-	local opt__verify_project="$opt_common $opt_mani $opt_lock"
 	local opt__version="$opt_common $opt_lock"
-	local opt__yank="$opt_common $opt_lock --vers --undo --index --token --registry"
+	local opt__yank="$opt_common $opt_lock --version --undo --index --token --registry"
 	local opt__libtest="--help --include-ignored --ignored --test --bench --list --logfile --nocapture --test-threads --skip -q --quiet --exact --color --format"
 
 	if [[ $cword -gt $dd_i ]]; then
@@ -97,7 +108,8 @@ _cargo()
 		elif [[ "$cur" == +* ]]; then
 			COMPREPLY=( $( compgen -W "$(_toolchains)" -- "$cur" ) )
 		else
-			COMPREPLY=( $( compgen -W "$__cargo_commands" -- "$cur" ) )
+			_ensure_cargo_commands_cache_filled
+			COMPREPLY=( $( compgen -W "$__cargo_commands_cache" -- "$cur" ) )
 		fi
 	else
 		case "${prev}" in
@@ -128,15 +140,20 @@ _cargo()
 			--target)
 				COMPREPLY=( $( compgen -W "$(_get_targets)" -- "$cur" ) )
 				;;
-			--target-dir)
+			--target-dir|--path)
 				_filedir -d
 				;;
 			help)
-				COMPREPLY=( $( compgen -W "$__cargo_commands" -- "$cur" ) )
+				_ensure_cargo_commands_cache_filled
+				COMPREPLY=( $( compgen -W "$__cargo_commands_cache" -- "$cur" ) )
 				;;
 			*)
-				local opt_var=opt__${cmd//-/_}
-				if [[ -z "${!opt_var}" ]]; then
+				if [[ "$cmd" == "report" && "$prev" == future-incompat* ]]; then
+					local opt_var=opt__${cmd//-/_}__${prev//-/_}
+				else
+					local opt_var=opt__${cmd//-/_}
+				fi
+				if [[ -z "${!opt_var-}" ]]; then
 					# Fallback to filename completion.
 					_filedir
 				else
@@ -152,7 +169,12 @@ _cargo()
 } &&
 complete -F _cargo cargo
 
-__cargo_commands=$(cargo --list 2>/dev/null | awk 'NR>1 {print $1}')
+__cargo_commands_cache=
+_ensure_cargo_commands_cache_filled(){
+	if [[ -z $__cargo_commands_cache ]]; then
+		__cargo_commands_cache="$(cargo --list 2>/dev/null | awk 'NR>1 {print $1}')"
+	fi
+}
 
 _locate_manifest(){
 	cargo locate-project --message-format plain 2>/dev/null
@@ -186,12 +208,12 @@ _get_names_from_array()
 				line=${line##*=}
 				line=${line%%\"}
 				line=${line##*\"}
-				names+=($line)
+				names+=("$line")
 			fi
 		fi
 
 		last_line=$line
-	done < $manifest
+	done < "$manifest"
 	echo "${names[@]}"
 }
 
@@ -227,15 +249,11 @@ _get_examples(){
 }
 
 _get_targets(){
-	local result=()
-	local targets=$(rustup target list)
-	while read line
-	do
-		if [[ "$line" =~ default|installed ]]; then
-			result+=("${line%% *}")
-		fi
-	done <<< "$targets"
-	echo "${result[@]}"
+	if command -v rustup >/dev/null 2>/dev/null; then
+	  rustup target list --installed
+	else
+	  rustc --print target-list
+	fi
 }
 
 _toolchains(){
