@@ -5,10 +5,11 @@ use std::ffi::OsStr;
 use std::path::PathBuf;
 
 use cargo_test_support::git;
+use cargo_test_support::git::cargo_uses_gitoxide;
 use cargo_test_support::paths;
+use cargo_test_support::prelude::*;
 use cargo_test_support::project;
 use cargo_test_support::registry::Package;
-
 use url::Url;
 
 pub fn find_index() -> PathBuf {
@@ -89,13 +90,18 @@ fn run_test(path_env: Option<&OsStr>) {
     );
 }
 
-#[cargo_test(requires_git)]
+#[cargo_test(requires = "git")]
 fn use_git_gc() {
     run_test(None);
 }
 
 #[cargo_test]
 fn avoid_using_git() {
+    if cargo_uses_gitoxide() {
+        // file protocol without git binary is currently not possible - needs built-in upload-pack.
+        // See https://github.com/Byron/gitoxide/issues/734 (support for the file protocol) progress updates.
+        return;
+    }
     let path = env::var_os("PATH").unwrap_or_default();
     let mut paths = env::split_paths(&path).collect::<Vec<_>>();
     let idx = paths
