@@ -1,7 +1,8 @@
 use cargo::core::compiler::Lto;
+use cargo_test_support::prelude::*;
 use cargo_test_support::registry::Package;
-use cargo_test_support::{basic_manifest, project, Project};
-use std::process::Output;
+use cargo_test_support::RawOutput;
+use cargo_test_support::{basic_manifest, project, str, Project};
 
 #[cargo_test]
 fn with_deps() {
@@ -14,6 +15,7 @@ fn with_deps() {
                 [package]
                 name = "test"
                 version = "0.0.0"
+                edition = "2015"
 
                 [dependencies]
                 bar = "*"
@@ -25,8 +27,18 @@ fn with_deps() {
         .file("src/main.rs", "extern crate bar; fn main() {}")
         .build();
     p.cargo("build -v --release")
-        .with_stderr_contains("[..]`rustc[..]--crate-name bar[..]-C linker-plugin-lto[..]`")
-        .with_stderr_contains("[..]`rustc[..]--crate-name test[..]-C lto[..]`")
+        .with_stderr_data(str![[r#"
+[UPDATING] `dummy-registry` index
+[LOCKING] 1 package to latest compatible version
+[DOWNLOADING] crates ...
+[DOWNLOADED] bar v0.0.1 (registry `dummy-registry`)
+[COMPILING] bar v0.0.1
+[RUNNING] `rustc --crate-name bar [..]-C linker-plugin-lto [..]`
+[COMPILING] test v0.0.0 ([ROOT]/foo)
+[RUNNING] `rustc --crate-name test [..]-C lto [..]`
+[FINISHED] `release` profile [optimized] target(s) in [ELAPSED]s
+
+"#]])
         .run();
 }
 
@@ -41,6 +53,7 @@ fn shared_deps() {
                 [package]
                 name = "test"
                 version = "0.0.0"
+                edition = "2015"
 
                 [dependencies]
                 bar = "*"
@@ -56,7 +69,24 @@ fn shared_deps() {
         .file("src/main.rs", "extern crate bar; fn main() {}")
         .build();
     p.cargo("build -v --release")
-        .with_stderr_contains("[..]`rustc[..]--crate-name test[..]-C lto[..]`")
+        .with_stderr_data(
+            str![[r#"
+[UPDATING] `dummy-registry` index
+[LOCKING] 1 package to latest compatible version
+[DOWNLOADING] crates ...
+[DOWNLOADED] bar v0.0.1 (registry `dummy-registry`)
+[COMPILING] bar v0.0.1
+[RUNNING] `rustc --crate-name bar [..]-C linker-plugin-lto [..]`
+[RUNNING] `rustc --crate-name bar [..]-C embed-bitcode=no [..]`
+[COMPILING] test v0.0.0 ([ROOT]/foo)
+[RUNNING] `rustc --crate-name build_script_build [..]`
+[RUNNING] `[ROOT]/foo/target/release/build/test-[HASH]/build-script-build`
+[RUNNING] `rustc --crate-name test [..]-C lto [..]`
+[FINISHED] `release` profile [optimized] target(s) in [ELAPSED]s
+
+"#]]
+            .unordered(),
+        )
         .run();
 }
 
@@ -71,6 +101,7 @@ fn build_dep_not_ltod() {
                 [package]
                 name = "test"
                 version = "0.0.0"
+                edition = "2015"
 
                 [build-dependencies]
                 bar = "*"
@@ -83,8 +114,20 @@ fn build_dep_not_ltod() {
         .file("src/main.rs", "fn main() {}")
         .build();
     p.cargo("build -v --release")
-        .with_stderr_contains("[..]`rustc[..]--crate-name bar[..]-C embed-bitcode=no[..]`")
-        .with_stderr_contains("[..]`rustc[..]--crate-name test[..]-C lto[..]`")
+        .with_stderr_data(str![[r#"
+[UPDATING] `dummy-registry` index
+[LOCKING] 1 package to latest compatible version
+[DOWNLOADING] crates ...
+[DOWNLOADED] bar v0.0.1 (registry `dummy-registry`)
+[COMPILING] bar v0.0.1
+[RUNNING] `rustc --crate-name bar [..]-C embed-bitcode=no [..]`
+[COMPILING] test v0.0.0 ([ROOT]/foo)
+[RUNNING] `rustc --crate-name build_script_build [..]`
+[RUNNING] `[ROOT]/foo/target/release/build/test-[HASH]/build-script-build`
+[RUNNING] `rustc --crate-name test [..]-C lto [..]`
+[FINISHED] `release` profile [optimized] target(s) in [ELAPSED]s
+
+"#]])
         .run();
 }
 
@@ -155,6 +198,7 @@ fn complicated() {
                 [package]
                 name = "test"
                 version = "0.0.0"
+                edition = "2015"
 
                 [lib]
                 crate-type = ['cdylib', 'staticlib']
@@ -232,6 +276,7 @@ fn off_in_manifest_works() {
                 [package]
                 name = "test"
                 version = "0.0.0"
+                edition = "2015"
 
                 [dependencies]
                 bar = "*"
@@ -250,19 +295,19 @@ fn off_in_manifest_works() {
         )
         .build();
     p.cargo("build -v --release")
-        .with_stderr(
-            "\
-[UPDATING] [..]
-[DOWNLOADING] [..]
-[DOWNLOADED] [..]
+        .with_stderr_data(str![[r#"
+[UPDATING] `dummy-registry` index
+[LOCKING] 1 package to latest compatible version
+[DOWNLOADING] crates ...
+[DOWNLOADED] bar v0.0.1 (registry `dummy-registry`)
 [COMPILING] bar v0.0.1
-[RUNNING] `rustc --crate-name bar [..]--crate-type lib [..]-C lto=off -C embed-bitcode=no[..]
-[COMPILING] test [..]
-[RUNNING] `rustc --crate-name test [..]--crate-type lib [..]-C lto=off -C embed-bitcode=no[..]
-[RUNNING] `rustc --crate-name test src/main.rs [..]--crate-type bin [..]-C lto=off[..]
-[FINISHED] [..]
-",
-        )
+[RUNNING] `rustc --crate-name bar [..]--crate-type lib [..]-C lto=off [..]-C embed-bitcode=no [..]`
+[COMPILING] test v0.0.0 ([ROOT]/foo)
+[RUNNING] `rustc --crate-name test [..]--crate-type lib [..]-C lto=off [..]-C embed-bitcode=no [..]`
+[RUNNING] `rustc --crate-name test --edition=2015 src/main.rs [..]--crate-type bin [..]-C lto=off [..]`
+[FINISHED] `release` profile [optimized] target(s) in [ELAPSED]s
+
+"#]])
         .run();
 }
 
@@ -275,6 +320,7 @@ fn between_builds() {
                 [package]
                 name = "test"
                 version = "0.0.0"
+                edition = "2015"
 
                 [profile.release]
                 lto = true
@@ -284,22 +330,20 @@ fn between_builds() {
         .file("src/main.rs", "fn main() { test::foo() }")
         .build();
     p.cargo("build -v --release --lib")
-        .with_stderr(
-            "\
-[COMPILING] test [..]
-[RUNNING] `rustc [..]--crate-type lib[..]-C linker-plugin-lto[..]
-[FINISHED] [..]
-",
-        )
+        .with_stderr_data(str![[r#"
+[COMPILING] test v0.0.0 ([ROOT]/foo)
+[RUNNING] `rustc [..]--crate-type lib [..]-C linker-plugin-lto [..]`
+[FINISHED] `release` profile [optimized] target(s) in [ELAPSED]s
+
+"#]])
         .run();
     p.cargo("build -v --release")
-        .with_stderr_contains(
-            "\
-[COMPILING] test [..]
-[RUNNING] `rustc [..]--crate-type bin[..]-C lto[..]
-[FINISHED] [..]
-",
-        )
+        .with_stderr_data(str![[r#"
+[COMPILING] test v0.0.0 ([ROOT]/foo)
+[RUNNING] `rustc [..]--crate-type bin [..]-C lto [..]`
+[FINISHED] `release` profile [optimized] target(s) in [ELAPSED]s
+
+"#]])
         .run();
 }
 
@@ -312,6 +356,7 @@ fn test_all() {
                 [package]
                 name = "foo"
                 version = "0.0.0"
+                edition = "2015"
 
                 [profile.release]
                 lto = true
@@ -322,7 +367,21 @@ fn test_all() {
         .file("tests/b.rs", "")
         .build();
     p.cargo("test --release -v")
-        .with_stderr_contains("[RUNNING] `rustc[..]--crate-name foo[..]-C lto[..]")
+        .with_stderr_data(
+            str![[r#"
+[COMPILING] foo v0.0.0 ([ROOT]/foo)
+[RUNNING] `rustc --crate-name a [..]-C lto [..]`
+[RUNNING] `rustc --crate-name b [..]-C lto [..]`
+[RUNNING] `rustc --crate-name foo [..]-C lto [..]--test [..]`
+[RUNNING] `rustc --crate-name foo [..]--crate-type bin [..]-C lto [..]`
+[FINISHED] `release` profile [optimized] target(s) in [ELAPSED]s
+[RUNNING] `[ROOT]/foo/target/release/deps/foo-[HASH][EXE]`
+[RUNNING] `[ROOT]/foo/target/release/deps/a-[HASH][EXE]`
+[RUNNING] `[ROOT]/foo/target/release/deps/b-[HASH][EXE]`
+
+"#]]
+            .unordered(),
+        )
         .run();
 }
 
@@ -335,6 +394,7 @@ fn test_all_and_bench() {
                 [package]
                 name = "foo"
                 version = "0.0.0"
+                edition = "2015"
 
                 [profile.release]
                 lto = true
@@ -347,9 +407,21 @@ fn test_all_and_bench() {
         .file("tests/b.rs", "")
         .build();
     p.cargo("test --release -v")
-        .with_stderr_contains("[RUNNING] `rustc[..]--crate-name a[..]-C lto[..]")
-        .with_stderr_contains("[RUNNING] `rustc[..]--crate-name b[..]-C lto[..]")
-        .with_stderr_contains("[RUNNING] `rustc[..]--crate-name foo[..]-C lto[..]")
+        .with_stderr_data(
+            str![[r#"
+[COMPILING] foo v0.0.0 ([ROOT]/foo)
+[RUNNING] `rustc --crate-name a [..]-C lto [..]`
+[RUNNING] `rustc --crate-name b [..]-C lto [..]`
+[RUNNING] `rustc --crate-name foo [..]-C lto [..]--test [..]`
+[RUNNING] `rustc --crate-name foo [..]--crate-type bin [..]-C lto [..]`
+[FINISHED] `release` profile [optimized] target(s) in [ELAPSED]s
+[RUNNING] `[ROOT]/foo/target/release/deps/foo-[HASH][EXE]`
+[RUNNING] `[ROOT]/foo/target/release/deps/a-[HASH][EXE]`
+[RUNNING] `[ROOT]/foo/target/release/deps/b-[HASH][EXE]`
+
+"#]]
+            .unordered(),
+        )
         .run();
 }
 
@@ -377,6 +449,7 @@ fn project_with_dep(crate_types: &str) -> Project {
                 [package]
                 name = "foo"
                 version = "0.0.0"
+                edition = "2015"
 
                 [workspace]
 
@@ -404,6 +477,7 @@ fn project_with_dep(crate_types: &str) -> Project {
                     [package]
                     name = "bar"
                     version = "0.0.0"
+                    edition = "2015"
 
                     [dependencies]
                     registry = "*"
@@ -434,7 +508,7 @@ fn project_with_dep(crate_types: &str) -> Project {
 ///
 /// `krate_info` is extra compiler flags used to distinguish this if the same
 /// crate name is being built multiple times.
-fn verify_lto(output: &Output, krate: &str, krate_info: &str, expected_lto: Lto) {
+fn verify_lto(output: &RawOutput, krate: &str, krate_info: &str, expected_lto: Lto) {
     let stderr = std::str::from_utf8(&output.stderr).unwrap();
     let mut matches = stderr.lines().filter(|line| {
         line.contains("Running")
@@ -453,10 +527,8 @@ fn verify_lto(output: &Output, krate: &str, krate_info: &str, expected_lto: Lto)
             krate, krate_info, line, line2, stderr
         );
     }
-    let actual_lto = if let Some(index) = line.find("-C lto=") {
-        let s = &line[index..];
-        let end = s.find(' ').unwrap();
-        let mode = &line[index..index + end];
+    let actual_lto = if let Some((_, line)) = line.split_once("-C lto=") {
+        let mode = line.splitn(2, ' ').next().unwrap();
         if mode == "off" {
             Lto::Off
         } else {
@@ -481,7 +553,7 @@ fn verify_lto(output: &Output, krate: &str, krate_info: &str, expected_lto: Lto)
 #[cargo_test]
 fn cdylib_and_rlib() {
     let p = project_with_dep("'cdylib', 'rlib'");
-    let output = p.cargo("build --release -v").exec_with_output().unwrap();
+    let output = p.cargo("build --release -v").run();
     // `registry` is ObjectAndBitcode because it needs Object for the
     // rlib, and Bitcode for the cdylib (which doesn't support LTO).
     verify_lto(
@@ -506,52 +578,55 @@ fn cdylib_and_rlib() {
     );
     verify_lto(&output, "foo", "--crate-type bin", Lto::Run(None));
     p.cargo("test --release -v")
-        .with_stderr_unordered(
-            "\
+        .with_stderr_data(
+            str![[r#"
 [FRESH] registry v0.0.1
 [FRESH] registry-shared v0.0.1
-[FRESH] bar v0.0.0 [..]
-[COMPILING] foo [..]
-[RUNNING] `rustc --crate-name foo [..]-C lto [..]--test[..]
-[RUNNING] `rustc --crate-name a [..]-C lto [..]--test[..]
-[FINISHED] [..]
-[RUNNING] [..]
-[RUNNING] [..]
-",
+[FRESH] bar v0.0.0 ([ROOT]/foo/bar)
+[COMPILING] foo v0.0.0 ([ROOT]/foo)
+[RUNNING] `rustc --crate-name foo [..]-C lto [..]--test [..]`
+[RUNNING] `rustc --crate-name a [..]-C lto [..]--test [..]`
+[FINISHED] `release` profile [optimized] target(s) in [ELAPSED]s
+[RUNNING] `[ROOT]/foo/target/release/deps/foo-[HASH][EXE]`
+[RUNNING] `[ROOT]/foo/target/release/deps/a-[HASH][EXE]`
+
+"#]]
+            .unordered(),
         )
         .run();
     p.cargo("build --release -v --manifest-path bar/Cargo.toml")
-        .with_stderr_unordered(
-            "\
+        .with_stderr_data(
+            str![[r#"
 [FRESH] registry-shared v0.0.1
 [FRESH] registry v0.0.1
-[FRESH] bar v0.0.0 [..]
-[FINISHED] [..]
-",
+[FRESH] bar v0.0.0 ([ROOT]/foo/bar)
+[FINISHED] `release` profile [optimized] target(s) in [ELAPSED]s
+
+"#]]
+            .unordered(),
         )
         .run();
     p.cargo("test --release -v --manifest-path bar/Cargo.toml")
-        .with_stderr_unordered(
-            "\
+        .with_stderr_data(str![[r#"
 [FRESH] registry-shared v0.0.1
 [FRESH] registry v0.0.1
-[COMPILING] bar [..]
-[RUNNING] `rustc --crate-name bar [..]-C lto[..]--test[..]
-[RUNNING] `rustc --crate-name b [..]-C lto[..]--test[..]
-[FINISHED] [..]
-[RUNNING] [..]target/release/deps/bar-[..]
-[RUNNING] [..]target/release/deps/b-[..]
+[COMPILING] bar v0.0.0 ([ROOT]/foo/bar)
+[RUNNING] `rustc --crate-name bar [..]-C lto [..]--test [..]`
+[RUNNING] `rustc --crate-name b [..]-C lto [..]--test [..]`
+[FINISHED] `release` profile [optimized] target(s) in [ELAPSED]s
+[RUNNING] `[ROOT]/foo/target/release/deps/bar-[HASH][EXE]`
+[RUNNING] `[ROOT]/foo/target/release/deps/b-[HASH][EXE]`
 [DOCTEST] bar
-[RUNNING] `rustdoc --crate-type cdylib --crate-type rlib --crate-name bar --test [..]-C lto[..]
-",
-        )
+[RUNNING] `rustdoc --edition=2015 --crate-type cdylib --crate-type rlib --color auto --crate-name bar --test [..]-C lto [..]
+
+"#]].unordered())
         .run();
 }
 
 #[cargo_test]
 fn dylib() {
     let p = project_with_dep("'dylib'");
-    let output = p.cargo("build --release -v").exec_with_output().unwrap();
+    let output = p.cargo("build --release -v").run();
     // `registry` is OnlyObject because rustc doesn't support LTO with dylibs.
     verify_lto(&output, "registry", "--crate-type lib", Lto::OnlyObject);
     // `registry_shared` is both because it is needed by both bar (Object) and
@@ -570,18 +645,20 @@ fn dylib() {
     // executables with `lto=true` because the tests are built with the
     // `--release` flag.
     p.cargo("test --release -v")
-        .with_stderr_unordered(
-            "\
+        .with_stderr_data(
+            str![[r#"
 [FRESH] registry v0.0.1
 [FRESH] registry-shared v0.0.1
-[FRESH] bar v0.0.0 [..]
-[COMPILING] foo [..]
-[RUNNING] `rustc --crate-name foo [..]-C lto [..]--test[..]
-[RUNNING] `rustc --crate-name a [..]-C lto [..]--test[..]
-[FINISHED] [..]
-[RUNNING] [..]
-[RUNNING] [..]
-",
+[FRESH] bar v0.0.0 ([ROOT]/foo/bar)
+[COMPILING] foo v0.0.0 ([ROOT]/foo)
+[RUNNING] `rustc --crate-name foo [..]-C lto [..]--test [..]`
+[RUNNING] `rustc --crate-name a [..]-C lto [..]--test [..]`
+[FINISHED] `release` profile [optimized] target(s) in [ELAPSED]s
+[RUNNING] `[ROOT]/foo/target/release/deps/foo-[HASH][EXE]`
+[RUNNING] `[ROOT]/foo/target/release/deps/a-[HASH][EXE]`
+
+"#]]
+            .unordered(),
         )
         .run();
     // Building just `bar` causes `registry-shared` to get rebuilt because it
@@ -590,16 +667,18 @@ fn dylib() {
     //
     // `bar` gets rebuilt because `registry_shared` got rebuilt.
     p.cargo("build --release -v --manifest-path bar/Cargo.toml")
-        .with_stderr_unordered(
-            "\
+        .with_stderr_data(
+            str![[r#"
 [COMPILING] registry-shared v0.0.1
 [FRESH] registry v0.0.1
-[RUNNING] `rustc --crate-name registry_shared [..]-C embed-bitcode=no[..]
+[RUNNING] `rustc --crate-name registry_shared [..]-C embed-bitcode=no [..]`
 [DIRTY] bar v0.0.0 ([..]): dependency info changed
-[COMPILING] bar [..]
-[RUNNING] `rustc --crate-name bar [..]--crate-type dylib [..]-C embed-bitcode=no[..]
-[FINISHED] [..]
-",
+[COMPILING] bar v0.0.0 ([ROOT]/foo/bar)
+[RUNNING] `rustc --crate-name bar [..]--crate-type dylib [..]-C embed-bitcode=no [..]`
+[FINISHED] `release` profile [optimized] target(s) in [ELAPSED]s
+
+"#]]
+            .unordered(),
         )
         .run();
     // Testing just `bar` causes `registry` to get rebuilt because it switches
@@ -608,20 +687,22 @@ fn dylib() {
     //
     // `bar` the dylib gets rebuilt because `registry` got rebuilt.
     p.cargo("test --release -v --manifest-path bar/Cargo.toml")
-        .with_stderr_unordered(
-            "\
+        .with_stderr_data(
+            str![[r#"
 [FRESH] registry-shared v0.0.1
 [COMPILING] registry v0.0.1
-[RUNNING] `rustc --crate-name registry [..]
+[RUNNING] `rustc --crate-name registry [..]`
 [DIRTY] bar v0.0.0 ([..]): dependency info changed
-[COMPILING] bar [..]
-[RUNNING] `rustc --crate-name bar [..]--crate-type dylib [..]-C embed-bitcode=no[..]
-[RUNNING] `rustc --crate-name bar [..]-C lto [..]--test[..]
-[RUNNING] `rustc --crate-name b [..]-C lto [..]--test[..]
-[FINISHED] [..]
-[RUNNING] [..]
-[RUNNING] [..]
-",
+[COMPILING] bar v0.0.0 ([ROOT]/foo/bar)
+[RUNNING] `rustc --crate-name bar [..]--crate-type dylib [..]-C embed-bitcode=no [..]`
+[RUNNING] `rustc --crate-name bar [..]-C lto [..]--test [..]`
+[RUNNING] `rustc --crate-name b [..]-C lto [..]--test [..]`
+[FINISHED] `release` profile [optimized] target(s) in [ELAPSED]s
+[RUNNING] `[ROOT]/foo/target/release/deps/bar-[HASH][EXE]`
+[RUNNING] `[ROOT]/foo/target/release/deps/b-[HASH][EXE]`
+
+"#]]
+            .unordered(),
         )
         .run();
 }
@@ -666,20 +747,22 @@ fn test_profile() {
 
     p.cargo("test -v")
         // unordered because the two `foo` builds start in parallel
-        .with_stderr_unordered("\
-[UPDATING] [..]
-[DOWNLOADING] [..]
-[DOWNLOADED] [..]
+        .with_stderr_data(str![[r#"
+[UPDATING] `dummy-registry` index
+[LOCKING] 1 package to latest compatible version
+[DOWNLOADING] crates ...
+[DOWNLOADED] bar v0.0.1 (registry `dummy-registry`)
 [COMPILING] bar v0.0.1
-[RUNNING] `rustc --crate-name bar [..]crate-type lib[..]
-[COMPILING] foo [..]
-[RUNNING] `rustc --crate-name foo [..]--crate-type lib --emit=dep-info,metadata,link -C linker-plugin-lto[..]
-[RUNNING] `rustc --crate-name foo [..]--emit=dep-info,link -C lto=thin [..]--test[..]
-[FINISHED] [..]
-[RUNNING] [..]
+[RUNNING] `rustc --crate-name bar [..]--crate-type lib [..]`
+[COMPILING] foo v0.1.0 ([ROOT]/foo)
+[RUNNING] `rustc --crate-name foo [..]--crate-type lib --emit=dep-info,metadata,link -C linker-plugin-lto [..]`
+[RUNNING] `rustc --crate-name foo [..]--emit=dep-info,link -C lto=thin [..]--test [..]`
+[FINISHED] `test` profile [unoptimized + debuginfo] target(s) in [ELAPSED]s
+[RUNNING] `[ROOT]/foo/target/debug/deps/foo-[HASH][EXE]`
 [DOCTEST] foo
 [RUNNING] `rustdoc [..]
-")
+
+"#]].unordered())
         .run();
 }
 
@@ -722,23 +805,33 @@ fn doctest() {
         .build();
 
     p.cargo("test --doc --release -v")
-        .with_stderr_contains("[..]`rustc --crate-name bar[..]-C linker-plugin-lto[..]")
-        .with_stderr_contains("[..]`rustc --crate-name foo[..]-C linker-plugin-lto[..]")
         // embed-bitcode should be harmless here
-        .with_stderr_contains("[..]`rustdoc [..]-C lto[..]")
+        .with_stderr_data(str![[r#"
+[LOCKING] 1 package to latest compatible version
+[COMPILING] bar v0.1.0 ([ROOT]/foo/bar)
+[RUNNING] `rustc --crate-name bar [..]--crate-type lib [..]-C linker-plugin-lto [..]`
+[COMPILING] foo v0.1.0 ([ROOT]/foo)
+[RUNNING] `rustc --crate-name foo [..]--crate-type lib [..]-C linker-plugin-lto [..]`
+[FINISHED] `release` profile [optimized] target(s) in [ELAPSED]s
+[DOCTEST] foo
+[RUNNING] `rustdoc [..]`
+
+"#]])
         .run();
 
     // Try with bench profile.
     p.cargo("test --doc --release -v")
         .env("CARGO_PROFILE_BENCH_LTO", "true")
-        .with_stderr_unordered(
-            "\
-[FRESH] bar v0.1.0 [..]
-[FRESH] foo v0.1.0 [..]
-[FINISHED] release [..]
+        .with_stderr_data(
+            str![[r#"
 [DOCTEST] foo
-[RUNNING] `rustdoc [..]-C lto[..]
-",
+[RUNNING] `rustdoc [..]-C lto [..]`
+[FRESH] bar v0.1.0 ([ROOT]/foo/bar)
+[FRESH] foo v0.1.0 ([ROOT]/foo)
+[FINISHED] `release` profile [optimized] target(s) in [ELAPSED]s
+
+"#]]
+            .unordered(),
         )
         .run();
 }
@@ -753,6 +846,7 @@ fn dylib_rlib_bin() {
                 [package]
                 name = "foo"
                 version = "0.1.0"
+                edition = "2015"
 
                 [lib]
                 crate-type = ["dylib", "rlib"]
@@ -765,7 +859,7 @@ fn dylib_rlib_bin() {
         .file("src/bin/ferret.rs", "fn main() { foo::foo(); }")
         .build();
 
-    let output = p.cargo("build --release -v").exec_with_output().unwrap();
+    let output = p.cargo("build --release -v").run();
     verify_lto(
         &output,
         "foo",
@@ -789,6 +883,7 @@ fn fresh_swapping_commands() {
                 [package]
                 name = "foo"
                 version = "0.1.0"
+                edition = "2015"
 
                 [dependencies]
                 bar = "1.0"
@@ -801,50 +896,50 @@ fn fresh_swapping_commands() {
         .build();
 
     p.cargo("build --release -v")
-        .with_stderr(
-            "\
-[UPDATING] [..]
+        .with_stderr_data(str![[r#"
+[UPDATING] `dummy-registry` index
+[LOCKING] 1 package to latest compatible version
 [DOWNLOADING] crates ...
-[DOWNLOADED] bar v1.0.0 [..]
+[DOWNLOADED] bar v1.0.0 (registry `dummy-registry`)
 [COMPILING] bar v1.0.0
-[RUNNING] `rustc --crate-name bar [..]-C linker-plugin-lto[..]
-[COMPILING] foo v0.1.0 [..]
-[RUNNING] `rustc --crate-name foo src/lib.rs [..]-C linker-plugin-lto[..]
-[FINISHED] [..]
-",
-        )
+[RUNNING] `rustc --crate-name bar [..]-C linker-plugin-lto [..]`
+[COMPILING] foo v0.1.0 ([ROOT]/foo)
+[RUNNING] `rustc --crate-name foo --edition=2015 src/lib.rs [..]-C linker-plugin-lto [..]`
+[FINISHED] `release` profile [optimized] target(s) in [ELAPSED]s
+
+"#]])
         .run();
     p.cargo("test --release -v")
-        .with_stderr_unordered(
-            "\
+        .with_stderr_data(
+            str![[r#"
 [FRESH] bar v1.0.0
-[COMPILING] foo v0.1.0 [..]
-[RUNNING] `rustc --crate-name foo src/lib.rs [..]-C lto[..]--test[..]
-[FINISHED] [..]
-[RUNNING] `[..]/foo[..]`
+[COMPILING] foo v0.1.0 ([ROOT]/foo)
+[RUNNING] `rustc --crate-name foo --edition=2015 src/lib.rs [..]-C lto [..]--test [..]`
+[FINISHED] `release` profile [optimized] target(s) in [ELAPSED]s
+[RUNNING] `[ROOT]/foo/target/release/deps/foo-[HASH][EXE]`
 [DOCTEST] foo
-[RUNNING] `rustdoc [..]-C lto[..]
-",
+[RUNNING] `rustdoc [..]-C lto [..]`
+
+"#]]
+            .unordered(),
         )
         .run();
 
     p.cargo("build --release -v")
-        .with_stderr(
-            "\
+        .with_stderr_data(str![[r#"
 [FRESH] bar v1.0.0
-[FRESH] foo [..]
-[FINISHED] [..]
-",
-        )
+[FRESH] foo v0.1.0 ([ROOT]/foo)
+[FINISHED] `release` profile [optimized] target(s) in [ELAPSED]s
+
+"#]])
         .run();
     p.cargo("test --release -v --no-run -v")
-        .with_stderr(
-            "\
+        .with_stderr_data(str![[r#"
 [FRESH] bar v1.0.0
-[FRESH] foo [..]
-[FINISHED] [..]
-[EXECUTABLE] `[..]/target/release/deps/foo-[..][EXE]`
-",
-        )
+[FRESH] foo v0.1.0 ([ROOT]/foo)
+[FINISHED] `release` profile [optimized] target(s) in [ELAPSED]s
+[EXECUTABLE] `[ROOT]/foo/target/release/deps/foo-[HASH][EXE]`
+
+"#]])
         .run();
 }
