@@ -1,7 +1,8 @@
 //! Tests for profile overrides (build-override and per-package overrides).
 
+use cargo_test_support::prelude::*;
 use cargo_test_support::registry::Package;
-use cargo_test_support::{basic_lib_manifest, basic_manifest, project};
+use cargo_test_support::{basic_lib_manifest, basic_manifest, project, str};
 
 #[cargo_test]
 fn profile_override_basic() {
@@ -12,6 +13,7 @@ fn profile_override_basic() {
                 [package]
                 name = "foo"
                 version = "0.0.1"
+                edition = "2015"
                 authors = []
 
                 [dependencies]
@@ -30,13 +32,15 @@ fn profile_override_basic() {
         .build();
 
     p.cargo("check -v")
-        .with_stderr(
-            "[CHECKING] bar [..]
+        .with_stderr_data(str![[r#"
+[LOCKING] 1 package to latest compatible version
+[CHECKING] bar v0.5.0 ([ROOT]/foo/bar)
 [RUNNING] `rustc --crate-name bar [..] -C opt-level=3 [..]`
-[CHECKING] foo [..]
+[CHECKING] foo v0.0.1 ([ROOT]/foo)
 [RUNNING] `rustc --crate-name foo [..] -C opt-level=1 [..]`
-[FINISHED] dev [optimized + debuginfo] target(s) in [..]",
-        )
+[FINISHED] `dev` profile [optimized + debuginfo] target(s) in [ELAPSED]s
+
+"#]])
         .run();
 }
 
@@ -49,6 +53,7 @@ fn profile_override_warnings() {
                 [package]
                 name = "foo"
                 version = "0.0.1"
+                edition = "2015"
 
                 [dependencies]
                 bar = {path = "bar"}
@@ -68,20 +73,18 @@ fn profile_override_warnings() {
         .file("bar/src/lib.rs", "")
         .build();
 
-    p.cargo("build")
-        .with_stderr_contains(
-            "\
-[WARNING] profile package spec `bar@1.2.3` in profile `dev` \
-    has a version or URL that does not match any of the packages: \
-    bar v0.5.0 ([..]/foo/bar)
+    p.cargo("build").with_stderr_data(str![[r#"
+...
+[WARNING] profile package spec `bar@1.2.3` in profile `dev` has a version or URL that does not match any of the packages: bar v0.5.0 ([ROOT]/foo/bar)
 [WARNING] profile package spec `bart` in profile `dev` did not match any packages
 
-<tab>Did you mean `bar`?
+	Did you mean `bar`?
 [WARNING] profile package spec `no-suggestion` in profile `dev` did not match any packages
-[COMPILING] [..]
-",
-        )
-        .run();
+[COMPILING] bar v0.5.0 ([ROOT]/foo/bar)
+[COMPILING] foo v0.0.1 ([ROOT]/foo)
+[FINISHED] `dev` profile [unoptimized + debuginfo] target(s) in [ELAPSED]s
+
+"#]]).run();
 }
 
 #[cargo_test]
@@ -110,6 +113,7 @@ fn profile_override_bad_settings() {
                         [package]
                         name = "foo"
                         version = "0.0.1"
+                        edition = "2015"
 
                         [dependencies]
                         bar = {{path = "bar"}}
@@ -127,7 +131,13 @@ fn profile_override_bad_settings() {
 
         p.cargo("check")
             .with_status(101)
-            .with_stderr_contains(format!("Caused by:\n  {}", expected))
+            .with_stderr_data(format!(
+                "\
+...
+Caused by:\n  {}
+",
+                expected
+            ))
             .run();
     }
 }
@@ -162,6 +172,7 @@ fn profile_override_hierarchy() {
             [package]
             name = "m1"
             version = "0.0.1"
+            edition = "2015"
 
             [dependencies]
             m2 = { path = "../m2" }
@@ -177,6 +188,7 @@ fn profile_override_hierarchy() {
             [package]
             name = "m2"
             version = "0.0.1"
+            edition = "2015"
 
             [dependencies]
             m3 = { path = "../m3" }
@@ -212,23 +224,25 @@ fn profile_override_hierarchy() {
     // m2: 2 (as [profile.dev.package.m2])
     // m1: 1 (as [profile.dev])
 
-    p.cargo("build -v").with_stderr_unordered("\
-[COMPILING] m3 [..]
-[COMPILING] dep [..]
-[RUNNING] `rustc --crate-name m3 m3/src/lib.rs [..] --crate-type lib --emit=[..]link[..]-C codegen-units=4 [..]
-[RUNNING] `rustc --crate-name dep [..]dep/src/lib.rs [..] --crate-type lib --emit=[..]link[..]-C codegen-units=3 [..]
-[RUNNING] `rustc --crate-name m3 m3/src/lib.rs [..] --crate-type lib --emit=[..]link[..]-C codegen-units=1 [..]
-[RUNNING] `rustc --crate-name build_script_build m1/build.rs [..] --crate-type bin --emit=[..]link[..]-C codegen-units=4 [..]
-[COMPILING] m2 [..]
-[RUNNING] `rustc --crate-name build_script_build m2/build.rs [..] --crate-type bin --emit=[..]link[..]-C codegen-units=2 [..]
-[RUNNING] `[..]/m1-[..]/build-script-build`
-[RUNNING] `[..]/m2-[..]/build-script-build`
-[RUNNING] `rustc --crate-name m2 m2/src/lib.rs [..] --crate-type lib --emit=[..]link[..]-C codegen-units=2 [..]
-[COMPILING] m1 [..]
-[RUNNING] `rustc --crate-name m1 m1/src/lib.rs [..] --crate-type lib --emit=[..]link[..]-C codegen-units=1 [..]
-[FINISHED] dev [unoptimized + debuginfo] [..]
-",
-        )
+    p.cargo("build -v")
+        .with_stderr_data(str![[r#"
+[LOCKING] 1 package to latest compatible version
+[COMPILING] m3 v0.5.0 ([ROOT]/foo/m3)
+[COMPILING] dep v0.5.0 ([ROOT]/dep)
+[RUNNING] `rustc --crate-name m3 --edition=2015 m3/src/lib.rs [..] --crate-type lib --emit=[..]link[..]-C codegen-units=4 [..]`
+[RUNNING] `rustc --crate-name dep [..][ROOT]/dep/src/lib.rs [..] --crate-type lib --emit=[..]link[..]-C codegen-units=3 [..]`
+[RUNNING] `rustc --crate-name m3 --edition=2015 m3/src/lib.rs [..] --crate-type lib --emit=[..]link[..]-C codegen-units=1 [..]`
+[RUNNING] `rustc --crate-name build_script_build --edition=2015 m1/build.rs [..] --crate-type bin --emit=[..]link[..]-C codegen-units=4 [..]`
+[COMPILING] m2 v0.0.1 ([ROOT]/foo/m2)
+[RUNNING] `rustc --crate-name build_script_build --edition=2015 m2/build.rs [..] --crate-type bin --emit=[..]link[..]-C codegen-units=2 [..]`
+[RUNNING] `[ROOT]/foo/target/debug/build/m1-[HASH]/build-script-build`
+[RUNNING] `[ROOT]/foo/target/debug/build/m2-[HASH]/build-script-build`
+[RUNNING] `rustc --crate-name m2 --edition=2015 m2/src/lib.rs [..] --crate-type lib --emit=[..]link[..]-C codegen-units=2 [..]`
+[COMPILING] m1 v0.0.1 ([ROOT]/foo/m1)
+[RUNNING] `rustc --crate-name m1 --edition=2015 m1/src/lib.rs [..] --crate-type lib --emit=[..]link[..]-C codegen-units=1 [..]`
+[FINISHED] `dev` profile [unoptimized + debuginfo] target(s) in [ELAPSED]s
+
+"#]].unordered())
         .run();
 }
 
@@ -241,6 +255,7 @@ fn profile_override_spec_multiple() {
             [package]
             name = "foo"
             version = "0.0.1"
+            edition = "2015"
 
             [dependencies]
             bar = { path = "bar" }
@@ -259,11 +274,78 @@ fn profile_override_spec_multiple() {
 
     p.cargo("check -v")
         .with_status(101)
-        .with_stderr_contains(
-            "\
-[ERROR] multiple package overrides in profile `dev` match package `bar v0.5.0 ([..])`
-found package specs: bar, bar@0.5.0",
+        .with_stderr_data(str![[r#"
+...
+[ERROR] multiple package overrides in profile `dev` match package `bar v0.5.0 ([ROOT]/foo/bar)`
+found package specs: bar, bar@0.5.0
+
+"#]])
+        .run();
+}
+
+#[cargo_test]
+fn profile_override_spec_with_version() {
+    let p = project()
+        .file(
+            "Cargo.toml",
+            r#"
+            [package]
+            name = "foo"
+            version = "0.0.1"
+            edition = "2015"
+
+            [dependencies]
+            bar = { path = "bar" }
+
+            [profile.dev.package."bar:0.5.0"]
+            codegen-units = 2
+            "#,
         )
+        .file("src/lib.rs", "")
+        .file("bar/Cargo.toml", &basic_lib_manifest("bar"))
+        .file("bar/src/lib.rs", "")
+        .build();
+
+    p.cargo("check -v")
+        .with_stderr_data(str![[r#"
+...
+[CHECKING] bar v0.5.0 ([ROOT]/foo/bar)
+[RUNNING] `rustc [..]bar/src/lib.rs [..] -C codegen-units=2 [..]`
+...
+"#]])
+        .run();
+}
+
+#[cargo_test]
+fn profile_override_spec_with_partial_version() {
+    let p = project()
+        .file(
+            "Cargo.toml",
+            r#"
+            [package]
+            name = "foo"
+            version = "0.0.1"
+            edition = "2015"
+
+            [dependencies]
+            bar = { path = "bar" }
+
+            [profile.dev.package."bar:0.5"]
+            codegen-units = 2
+            "#,
+        )
+        .file("src/lib.rs", "")
+        .file("bar/Cargo.toml", &basic_lib_manifest("bar"))
+        .file("bar/src/lib.rs", "")
+        .build();
+
+    p.cargo("check -v")
+        .with_stderr_data(str![[r#"
+...
+[CHECKING] bar v0.5.0 ([ROOT]/foo/bar)
+[RUNNING] `rustc [..]bar/src/lib.rs [..] -C codegen-units=2 [..]`
+...
+"#]])
         .run();
 }
 
@@ -290,6 +372,7 @@ fn profile_override_spec() {
             [package]
             name = "m1"
             version = "0.0.1"
+            edition = "2015"
 
             [dependencies]
             dep = { path = "../../dep1" }
@@ -303,6 +386,7 @@ fn profile_override_spec() {
             [package]
             name = "m2"
             version = "0.0.1"
+            edition = "2015"
 
             [dependencies]
             dep = {path = "../../dep2" }
@@ -324,8 +408,15 @@ fn profile_override_spec() {
         .build();
 
     p.cargo("check -v")
-        .with_stderr_contains("[RUNNING] `rustc [..]dep1/src/lib.rs [..] -C codegen-units=1 [..]")
-        .with_stderr_contains("[RUNNING] `rustc [..]dep2/src/lib.rs [..] -C codegen-units=2 [..]")
+        .with_stderr_data(
+            str![[r#"
+...
+[RUNNING] `rustc [..][ROOT]/dep1/src/lib.rs [..] -C codegen-units=1 [..]`
+[RUNNING] `rustc [..][ROOT]/dep2/src/lib.rs [..] -C codegen-units=2 [..]`
+...
+"#]]
+            .unordered(),
+        )
         .run();
 }
 
@@ -356,6 +447,7 @@ fn override_proc_macro() {
             [package]
             name = "pm"
             version = "0.1.0"
+            edition = "2015"
 
             [lib]
             proc-macro = true
@@ -380,13 +472,18 @@ fn override_proc_macro() {
 
     p.cargo("check -v")
         // Shared built for the proc-macro.
-        .with_stderr_contains("[RUNNING] `rustc [..]--crate-name shared [..]-C codegen-units=4[..]")
+        .with_stderr_data(str![[r#"
+...
+[RUNNING] `rustc [..]--crate-name shared [..] -C codegen-units=4[..]`
+...
+[RUNNING] `rustc [..]--crate-name pm [..] -C codegen-units=4[..]`
+...
+"#]])
         // Shared built for the library.
         .with_stderr_line_without(
-            &["[RUNNING] `rustc --crate-name shared"],
+            &["[RUNNING] `rustc --crate-name shared --edition=2015"],
             &["-C codegen-units"],
         )
-        .with_stderr_contains("[RUNNING] `rustc [..]--crate-name pm [..]-C codegen-units=4[..]")
         .with_stderr_line_without(
             &["[RUNNING] `rustc [..]--crate-name foo"],
             &["-C codegen-units"],
@@ -415,12 +512,11 @@ fn no_warning_ws() {
         .build();
 
     p.cargo("check -p b")
-        .with_stderr(
-            "\
-[CHECKING] b [..]
-[FINISHED] [..]
-",
-        )
+        .with_stderr_data(str![[r#"
+[CHECKING] b v0.1.0 ([ROOT]/foo/b)
+[FINISHED] `dev` profile [unoptimized + debuginfo] target(s) in [ELAPSED]s
+
+"#]])
         .run();
 }
 
@@ -448,9 +544,9 @@ fn build_override_shared() {
             r#"
             fn main() {
                 if std::env::var("DEBUG").unwrap() != "false" {
-                    println!("cargo:rustc-cfg=foo_debug");
+                    println!("cargo::rustc-cfg=foo_debug");
                 } else {
-                    println!("cargo:rustc-cfg=foo_release");
+                    println!("cargo::rustc-cfg=foo_release");
                 }
             }
             "#,
