@@ -6,8 +6,11 @@
 
 use std::fs;
 
+use cargo_test_support::compare::assert_e2e;
 use cargo_test_support::git;
+use cargo_test_support::prelude::*;
 use cargo_test_support::registry::{self, Package, RegistryBuilder};
+use cargo_test_support::str;
 use cargo_test_support::{basic_lib_manifest, basic_manifest, paths, project, Project};
 
 #[cargo_test]
@@ -33,7 +36,7 @@ fn vendor_simple() {
     let lock = p.read_file("vendor/log/Cargo.toml");
     assert!(lock.contains("version = \"0.3.5\""));
 
-    add_vendor_config(&p);
+    add_crates_io_vendor_config(&p);
     p.cargo("check").run();
 }
 
@@ -57,14 +60,14 @@ fn vendor_sample_config() {
     Package::new("log", "0.3.5").publish();
 
     p.cargo("vendor --respect-source-config")
-        .with_stdout(
-            r#"[source.crates-io]
+        .with_stdout_data(str![[r#"
+[source.crates-io]
 replace-with = "vendored-sources"
 
 [source.vendored-sources]
 directory = "vendor"
-"#,
-        )
+
+"#]])
         .run();
 }
 
@@ -88,9 +91,8 @@ fn vendor_sample_config_alt_registry() {
 
     Package::new("log", "0.3.5").alternative(true).publish();
 
-    p.cargo("vendor --respect-source-config -Z sparse-registry")
-        .masquerade_as_nightly_cargo(&["sparse-registry"])
-        .with_stdout(format!(
+    p.cargo("vendor --respect-source-config")
+        .with_stdout_data(format!(
             r#"[source."{0}"]
 registry = "{0}"
 replace-with = "vendored-sources"
@@ -128,11 +130,7 @@ fn vendor_path_specified() {
         "deps/.vendor"
     };
 
-    let output = p
-        .cargo("vendor --respect-source-config")
-        .arg(path)
-        .exec_with_output()
-        .unwrap();
+    let output = p.cargo("vendor --respect-source-config").arg(path).run();
     // Assert against original output to ensure that
     // path is normalized by `ops::vendor` on Windows.
     assert_eq!(
@@ -149,9 +147,9 @@ directory = "deps/.vendor"
     assert!(lock.contains("version = \"0.3.5\""));
 }
 
-fn add_vendor_config(p: &Project) {
+fn add_crates_io_vendor_config(p: &Project) {
     p.change_file(
-        ".cargo/config",
+        ".cargo/config.toml",
         r#"
             [source.crates-io]
             replace-with = 'vendor'
@@ -162,6 +160,23 @@ fn add_vendor_config(p: &Project) {
     );
 }
 
+fn add_git_vendor_config(p: &Project, git_project: &Project) {
+    p.change_file(
+        ".cargo/config.toml",
+        &format!(
+            r#"
+            [source."git+{url}"]
+            git = "{url}"
+            replace-with = 'vendor'
+
+            [source.vendor]
+            directory = 'vendor'
+        "#,
+            url = git_project.url()
+        ),
+    );
+}
+
 #[cargo_test]
 fn package_exclude() {
     let p = project()
@@ -206,6 +221,560 @@ fn package_exclude() {
     assert!(!csum.contains(".dotdir/include"));
 }
 
+#[cargo_test]
+fn discovery_inferred_build_rs_included() {
+    let git_project = git::new("dep", |project| {
+        project
+            .file(
+                "Cargo.toml",
+                r#"
+    [package]
+    name = "dep"
+    version = "0.0.1"
+    edition = "2015"
+    license = "MIT"
+    description = "foo"
+    documentation = "docs.rs/foo"
+    authors = []
+    include = ["src/lib.rs", "build.rs"]
+    "#,
+            )
+            .file("src/lib.rs", "")
+            .file("build.rs", "fn main() {}")
+    });
+
+    let p = project()
+        .file(
+            "Cargo.toml",
+            &format!(
+                r#"
+                    [package]
+                    name = "foo"
+                    version = "0.5.0"
+                    edition = "2015"
+
+                    [dependencies.dep]
+                    git = '{}'
+                "#,
+                git_project.url()
+            ),
+        )
+        .file("src/main.rs", "fn main() {}")
+        .build();
+
+    p.cargo("vendor --respect-source-config").run();
+    add_git_vendor_config(&p, &git_project);
+
+    let lock = p.read_file("vendor/dep/Cargo.toml");
+    assert_e2e().eq(
+        lock,
+        str![[r##"
+# THIS FILE IS AUTOMATICALLY GENERATED BY CARGO
+#
+# When uploading crates to the registry Cargo will automatically
+# "normalize" Cargo.toml files for maximal compatibility
+# with all versions of Cargo and also rewrite `path` dependencies
+# to registry (e.g., crates.io) dependencies.
+#
+# If you are reading this file be aware that the original Cargo.toml
+# will likely look very different (and much more reasonable).
+# See Cargo.toml.orig for the original contents.
+
+[package]
+edition = "2015"
+name = "dep"
+version = "0.0.1"
+authors = []
+build = "build.rs"
+include = [
+    "src/lib.rs",
+    "build.rs",
+]
+autolib = false
+autobins = false
+autoexamples = false
+autotests = false
+autobenches = false
+description = "foo"
+documentation = "docs.rs/foo"
+readme = false
+license = "MIT"
+
+[lib]
+name = "dep"
+path = "src/lib.rs"
+
+"##]],
+    );
+
+    p.cargo("check").run();
+}
+
+#[cargo_test]
+fn discovery_inferred_build_rs_excluded() {
+    let git_project = git::new("dep", |project| {
+        project
+            .file(
+                "Cargo.toml",
+                r#"
+    [package]
+    name = "dep"
+    version = "0.0.1"
+    edition = "2015"
+    license = "MIT"
+    description = "foo"
+    documentation = "docs.rs/foo"
+    authors = []
+    include = ["src/lib.rs"]
+    "#,
+            )
+            .file("src/lib.rs", "")
+            .file("build.rs", "fn main() {}")
+    });
+
+    let p = project()
+        .file(
+            "Cargo.toml",
+            &format!(
+                r#"
+                    [package]
+                    name = "foo"
+                    version = "0.5.0"
+                    edition = "2015"
+
+                    [dependencies.dep]
+                    git = '{}'
+                "#,
+                git_project.url()
+            ),
+        )
+        .file("src/main.rs", "fn main() {}")
+        .build();
+
+    p.cargo("vendor --respect-source-config").run();
+    add_git_vendor_config(&p, &git_project);
+
+    let lock = p.read_file("vendor/dep/Cargo.toml");
+    assert_e2e().eq(
+        lock,
+        str![[r##"
+# THIS FILE IS AUTOMATICALLY GENERATED BY CARGO
+#
+# When uploading crates to the registry Cargo will automatically
+# "normalize" Cargo.toml files for maximal compatibility
+# with all versions of Cargo and also rewrite `path` dependencies
+# to registry (e.g., crates.io) dependencies.
+#
+# If you are reading this file be aware that the original Cargo.toml
+# will likely look very different (and much more reasonable).
+# See Cargo.toml.orig for the original contents.
+
+[package]
+edition = "2015"
+name = "dep"
+version = "0.0.1"
+authors = []
+build = false
+include = ["src/lib.rs"]
+autolib = false
+autobins = false
+autoexamples = false
+autotests = false
+autobenches = false
+description = "foo"
+documentation = "docs.rs/foo"
+readme = false
+license = "MIT"
+
+[lib]
+name = "dep"
+path = "src/lib.rs"
+
+"##]],
+    );
+
+    p.cargo("check").run();
+}
+
+#[cargo_test]
+fn discovery_inferred_lib_included() {
+    let git_project = git::new("dep", |project| {
+        project
+            .file(
+                "Cargo.toml",
+                r#"
+    [package]
+    name = "dep"
+    version = "0.0.1"
+    edition = "2015"
+    license = "MIT"
+    description = "foo"
+    documentation = "docs.rs/foo"
+    authors = []
+    include = ["src/main.rs", "src/lib.rs"]
+    "#,
+            )
+            .file("src/main.rs", "fn main() {}")
+            .file("src/lib.rs", "")
+    });
+
+    let p = project()
+        .file(
+            "Cargo.toml",
+            &format!(
+                r#"
+                    [package]
+                    name = "foo"
+                    version = "0.5.0"
+                    edition = "2015"
+
+                    [dependencies.dep]
+                    git = '{}'
+                "#,
+                git_project.url()
+            ),
+        )
+        .file("src/main.rs", "fn main() {}")
+        .build();
+
+    p.cargo("vendor --respect-source-config").run();
+    add_git_vendor_config(&p, &git_project);
+
+    let lock = p.read_file("vendor/dep/Cargo.toml");
+    assert_e2e().eq(
+        lock,
+        str![[r##"
+# THIS FILE IS AUTOMATICALLY GENERATED BY CARGO
+#
+# When uploading crates to the registry Cargo will automatically
+# "normalize" Cargo.toml files for maximal compatibility
+# with all versions of Cargo and also rewrite `path` dependencies
+# to registry (e.g., crates.io) dependencies.
+#
+# If you are reading this file be aware that the original Cargo.toml
+# will likely look very different (and much more reasonable).
+# See Cargo.toml.orig for the original contents.
+
+[package]
+edition = "2015"
+name = "dep"
+version = "0.0.1"
+authors = []
+build = false
+include = [
+    "src/main.rs",
+    "src/lib.rs",
+]
+autolib = false
+autobins = false
+autoexamples = false
+autotests = false
+autobenches = false
+description = "foo"
+documentation = "docs.rs/foo"
+readme = false
+license = "MIT"
+
+[lib]
+name = "dep"
+path = "src/lib.rs"
+
+[[bin]]
+name = "dep"
+path = "src/main.rs"
+
+"##]],
+    );
+
+    p.cargo("check").run();
+}
+
+#[cargo_test]
+fn discovery_inferred_lib_excluded() {
+    let git_project = git::new("dep", |project| {
+        project
+            .file(
+                "Cargo.toml",
+                r#"
+    [package]
+    name = "dep"
+    version = "0.0.1"
+    edition = "2015"
+    license = "MIT"
+    description = "foo"
+    documentation = "docs.rs/foo"
+    authors = []
+    include = ["src/main.rs"]
+    "#,
+            )
+            .file("src/main.rs", "fn main() {}")
+            .file("src/lib.rs", "")
+    });
+
+    let p = project()
+        .file(
+            "Cargo.toml",
+            &format!(
+                r#"
+                    [package]
+                    name = "foo"
+                    version = "0.5.0"
+                    edition = "2015"
+
+                    [dependencies.dep]
+                    git = '{}'
+                "#,
+                git_project.url()
+            ),
+        )
+        .file("src/main.rs", "fn main() {}")
+        .build();
+
+    p.cargo("vendor --respect-source-config").run();
+    add_git_vendor_config(&p, &git_project);
+
+    let lock = p.read_file("vendor/dep/Cargo.toml");
+    assert_e2e().eq(
+        lock,
+        str![[r##"
+# THIS FILE IS AUTOMATICALLY GENERATED BY CARGO
+#
+# When uploading crates to the registry Cargo will automatically
+# "normalize" Cargo.toml files for maximal compatibility
+# with all versions of Cargo and also rewrite `path` dependencies
+# to registry (e.g., crates.io) dependencies.
+#
+# If you are reading this file be aware that the original Cargo.toml
+# will likely look very different (and much more reasonable).
+# See Cargo.toml.orig for the original contents.
+
+[package]
+edition = "2015"
+name = "dep"
+version = "0.0.1"
+authors = []
+build = false
+include = ["src/main.rs"]
+autolib = false
+autobins = false
+autoexamples = false
+autotests = false
+autobenches = false
+description = "foo"
+documentation = "docs.rs/foo"
+readme = false
+license = "MIT"
+
+[[bin]]
+name = "dep"
+path = "src/main.rs"
+
+"##]],
+    );
+
+    p.cargo("check").run();
+}
+
+#[cargo_test]
+fn discovery_inferred_other_included() {
+    let git_project = git::new("dep", |project| {
+        project
+        .file(
+            "Cargo.toml",
+            r#"
+    [package]
+    name = "dep"
+    version = "0.0.1"
+    edition = "2015"
+    license = "MIT"
+    description = "foo"
+    documentation = "docs.rs/foo"
+    authors = []
+    include = ["src/lib.rs", "src/bin/foo/main.rs", "examples/example_foo.rs", "tests/test_foo.rs", "benches/bench_foo.rs"]
+    "#,
+        )
+        .file("src/lib.rs", "")
+        .file("src/bin/foo/main.rs", "fn main() {}")
+        .file("examples/example_foo.rs", "fn main() {}")
+        .file("tests/test_foo.rs", "fn main() {}")
+        .file("benches/bench_foo.rs", "fn main() {}")
+    });
+
+    let p = project()
+        .file(
+            "Cargo.toml",
+            &format!(
+                r#"
+                    [package]
+                    name = "foo"
+                    version = "0.5.0"
+                    edition = "2015"
+
+                    [dependencies.dep]
+                    git = '{}'
+                "#,
+                git_project.url()
+            ),
+        )
+        .file("src/main.rs", "fn main() {}")
+        .build();
+
+    p.cargo("vendor --respect-source-config").run();
+    add_git_vendor_config(&p, &git_project);
+
+    let lock = p.read_file("vendor/dep/Cargo.toml");
+    assert_e2e().eq(
+        lock,
+        str![[r##"
+# THIS FILE IS AUTOMATICALLY GENERATED BY CARGO
+#
+# When uploading crates to the registry Cargo will automatically
+# "normalize" Cargo.toml files for maximal compatibility
+# with all versions of Cargo and also rewrite `path` dependencies
+# to registry (e.g., crates.io) dependencies.
+#
+# If you are reading this file be aware that the original Cargo.toml
+# will likely look very different (and much more reasonable).
+# See Cargo.toml.orig for the original contents.
+
+[package]
+edition = "2015"
+name = "dep"
+version = "0.0.1"
+authors = []
+build = false
+include = [
+    "src/lib.rs",
+    "src/bin/foo/main.rs",
+    "examples/example_foo.rs",
+    "tests/test_foo.rs",
+    "benches/bench_foo.rs",
+]
+autolib = false
+autobins = false
+autoexamples = false
+autotests = false
+autobenches = false
+description = "foo"
+documentation = "docs.rs/foo"
+readme = false
+license = "MIT"
+
+[lib]
+name = "dep"
+path = "src/lib.rs"
+
+[[bin]]
+name = "foo"
+path = "src/bin/foo/main.rs"
+
+[[example]]
+name = "example_foo"
+path = "examples/example_foo.rs"
+
+[[test]]
+name = "test_foo"
+path = "tests/test_foo.rs"
+
+[[bench]]
+name = "bench_foo"
+path = "benches/bench_foo.rs"
+
+"##]],
+    );
+
+    p.cargo("check").run();
+}
+
+#[cargo_test]
+fn discovery_inferred_other_excluded() {
+    let git_project = git::new("dep", |project| {
+        project
+            .file(
+                "Cargo.toml",
+                r#"
+    [package]
+    name = "dep"
+    version = "0.0.1"
+    edition = "2015"
+    license = "MIT"
+    description = "foo"
+    documentation = "docs.rs/foo"
+    authors = []
+    include = ["src/lib.rs"]
+    "#,
+            )
+            .file("src/lib.rs", "")
+            .file("src/bin/foo/main.rs", "fn main() {}")
+            .file("examples/example_foo.rs", "fn main() {}")
+            .file("tests/test_foo.rs", "fn main() {}")
+            .file("benches/bench_foo.rs", "fn main() {}")
+    });
+
+    let p = project()
+        .file(
+            "Cargo.toml",
+            &format!(
+                r#"
+                    [package]
+                    name = "foo"
+                    version = "0.5.0"
+                    edition = "2015"
+
+                    [dependencies.dep]
+                    git = '{}'
+                "#,
+                git_project.url()
+            ),
+        )
+        .file("src/main.rs", "fn main() {}")
+        .build();
+
+    p.cargo("vendor --respect-source-config").run();
+    add_git_vendor_config(&p, &git_project);
+
+    let lock = p.read_file("vendor/dep/Cargo.toml");
+    assert_e2e().eq(
+        lock,
+        str![[r##"
+# THIS FILE IS AUTOMATICALLY GENERATED BY CARGO
+#
+# When uploading crates to the registry Cargo will automatically
+# "normalize" Cargo.toml files for maximal compatibility
+# with all versions of Cargo and also rewrite `path` dependencies
+# to registry (e.g., crates.io) dependencies.
+#
+# If you are reading this file be aware that the original Cargo.toml
+# will likely look very different (and much more reasonable).
+# See Cargo.toml.orig for the original contents.
+
+[package]
+edition = "2015"
+name = "dep"
+version = "0.0.1"
+authors = []
+build = false
+include = ["src/lib.rs"]
+autolib = false
+autobins = false
+autoexamples = false
+autotests = false
+autobenches = false
+description = "foo"
+documentation = "docs.rs/foo"
+readme = false
+license = "MIT"
+
+[lib]
+name = "dep"
+path = "src/lib.rs"
+
+"##]],
+    );
+
+    p.cargo("check").run();
+}
+
 #[cargo_test]
 fn two_versions() {
     let p = project()
@@ -246,7 +815,7 @@ fn two_versions() {
     let lock = p.read_file("vendor/bitflags-0.7.0/Cargo.toml");
     assert!(lock.contains("version = \"0.7.0\""));
 
-    add_vendor_config(&p);
+    add_crates_io_vendor_config(&p);
     p.cargo("check").run();
 }
 
@@ -291,7 +860,7 @@ fn two_explicit_versions() {
     let lock = p.read_file("vendor/bitflags-0.7.0/Cargo.toml");
     assert!(lock.contains("version = \"0.7.0\""));
 
-    add_vendor_config(&p);
+    add_crates_io_vendor_config(&p);
     p.cargo("check").run();
 }
 
@@ -384,7 +953,7 @@ fn two_lockfiles() {
     let lock = p.read_file("vendor/bitflags-0.7.0/Cargo.toml");
     assert!(lock.contains("version = \"0.7.0\""));
 
-    add_vendor_config(&p);
+    add_crates_io_vendor_config(&p);
     p.cargo("check").cwd("foo").run();
     p.cargo("check").cwd("bar").run();
 }
@@ -435,12 +1004,14 @@ fn test_sync_argument() {
     Package::new("bitflags", "0.8.0").publish();
 
     p.cargo("vendor --respect-source-config --manifest-path foo/Cargo.toml -s bar/Cargo.toml baz/Cargo.toml test_vendor")
-        .with_stderr("\
-error: unexpected argument 'test_vendor' found
+        .with_stderr_data(str![[r#"
+[ERROR] unexpected argument 'test_vendor' found
 
 Usage: cargo[EXE] vendor [OPTIONS] [path]
 
-For more information, try '--help'.",
+For more information, try '--help'.
+
+"#]]
         )
         .with_status(1)
         .run();
@@ -641,7 +1212,7 @@ fn vendoring_git_crates() {
     p.cargo("vendor --respect-source-config").run();
     p.read_file("vendor/serde_derive/src/wut.rs");
 
-    add_vendor_config(&p);
+    add_crates_io_vendor_config(&p);
     p.cargo("check").run();
 }
 
@@ -711,21 +1282,21 @@ fn git_diff_rev() {
         .build();
 
     p.cargo("vendor --respect-source-config")
-        .with_stdout(
-            r#"[source."git+file://[..]/git?rev=v0.1.0"]
-git = [..]
+        .with_stdout_data(str![[r#"
+[source."git+[ROOTURL]/git?rev=v0.1.0"]
+git = "[ROOTURL]/git"
 rev = "v0.1.0"
 replace-with = "vendored-sources"
 
-[source."git+file://[..]/git?rev=v0.2.0"]
-git = [..]
+[source."git+[ROOTURL]/git?rev=v0.2.0"]
+git = "[ROOTURL]/git"
 rev = "v0.2.0"
 replace-with = "vendored-sources"
 
 [source.vendored-sources]
 directory = "vendor"
-"#,
-        )
+
+"#]])
         .run();
 }
 
@@ -770,21 +1341,21 @@ fn git_duplicate() {
     Package::new("b", "0.5.0").publish();
 
     p.cargo("vendor --respect-source-config")
-        .with_stderr(
-            "\
-[UPDATING] [..]
-[UPDATING] [..]
-[DOWNLOADING] [..]
-[DOWNLOADED] [..]
-error: failed to sync
+        .with_stderr_data(str![[r#"
+[UPDATING] git repository `[ROOTURL]/a`
+[UPDATING] `dummy-registry` index
+[LOCKING] 3 packages to latest compatible versions
+[DOWNLOADING] crates ...
+[DOWNLOADED] b v0.5.0 (registry `dummy-registry`)
+[ERROR] failed to sync
 
 Caused by:
   found duplicate version of package `b v0.5.0` vendored from two sources:
 
-  <tab>source 1: [..]
-  <tab>source 2: [..]
-",
-        )
+  	source 1: registry `crates-io`
+  	source 2: [ROOTURL]/a#[..]
+
+"#]])
         .with_status(101)
         .run();
 }
@@ -798,6 +1369,7 @@ fn git_complex() {
                 [package]
                 name = "b"
                 version = "0.1.0"
+                edition = "2021"
 
                 [dependencies]
                 dep_b = { path = 'dep_b' }
@@ -816,6 +1388,7 @@ fn git_complex() {
                     [package]
                     name = "a"
                     version = "0.1.0"
+                    edition = "2021"
 
                     [dependencies]
                     b = {{ git = '{}' }}
@@ -837,6 +1410,7 @@ fn git_complex() {
                     [package]
                     name = "foo"
                     version = "0.1.0"
+                    edition = "2021"
 
                     [dependencies]
                     a = {{ git = '{}' }}
@@ -847,21 +1421,211 @@ fn git_complex() {
         .file("src/lib.rs", "")
         .build();
 
-    let output = p
-        .cargo("vendor --respect-source-config")
-        .exec_with_output()
-        .unwrap();
+    let output = p.cargo("vendor --respect-source-config").run();
     let output = String::from_utf8(output.stdout).unwrap();
-    p.change_file(".cargo/config", &output);
+    p.change_file(".cargo/config.toml", &output);
 
     p.cargo("check -v")
-        .with_stderr_contains("[..]foo/vendor/a/src/lib.rs[..]")
-        .with_stderr_contains("[..]foo/vendor/dep_a/src/lib.rs[..]")
-        .with_stderr_contains("[..]foo/vendor/b/src/lib.rs[..]")
-        .with_stderr_contains("[..]foo/vendor/dep_b/src/lib.rs[..]")
+        .with_stderr_data(
+            str![[r#"
+[CHECKING] dep_b v0.5.0 ([ROOTURL]/git_b#[..])
+[CHECKING] dep_a v0.5.0 ([ROOTURL]/git_a#[..])
+[RUNNING] `rustc [..] [ROOT]/foo/vendor/dep_b/src/lib.rs [..]`
+[RUNNING] `rustc [..] [ROOT]/foo/vendor/dep_a/src/lib.rs [..]`
+[CHECKING] b v0.1.0 ([ROOTURL]/git_b#[..])
+[RUNNING] `rustc [..] [ROOT]/foo/vendor/b/src/lib.rs [..]`
+[CHECKING] a v0.1.0 ([ROOTURL]/git_a#[..])
+[RUNNING] `rustc [..] [ROOT]/foo/vendor/a/src/lib.rs [..]`
+[CHECKING] foo v0.1.0 ([ROOT]/foo)
+[RUNNING] `rustc [..] src/lib.rs [..]`
+[FINISHED] `dev` profile [unoptimized + debuginfo] target(s) in [ELAPSED]s
+
+"#]]
+            .unordered(),
+        )
         .run();
 }
 
+#[cargo_test]
+fn git_deterministic() {
+    let git_dep = git::new("git_dep", |p| {
+        p.file(
+            "Cargo.toml",
+            r#"
+            [package]
+            name = "git_dep"
+            version = "0.0.1"
+            edition = "2021"
+            license = "MIT"
+            description = "foo"
+            documentation = "docs.rs/foo"
+            authors = []
+
+            [[example]]
+            name = "c"
+
+            [[example]]
+            name = "b"
+
+            [[example]]
+            name = "a"
+            "#,
+        )
+        .file("src/lib.rs", "")
+        .file("examples/z.rs", "fn main() {}")
+        .file("examples/y.rs", "fn main() {}")
+        .file("examples/x.rs", "fn main() {}")
+        .file("examples/c.rs", "fn main() {}")
+        .file("examples/b.rs", "fn main() {}")
+        .file("examples/a.rs", "fn main() {}")
+    });
+
+    let p = project()
+        .file(
+            "Cargo.toml",
+            &format!(
+                r#"
+                    [package]
+                    name = "foo"
+                    version = "0.1.0"
+
+                    [dependencies]
+                    git_dep = {{ git = '{}' }}
+                "#,
+                git_dep.url()
+            ),
+        )
+        .file("src/lib.rs", "")
+        .build();
+
+    let output = p.cargo("vendor --respect-source-config").run();
+    let output = String::from_utf8(output.stdout).unwrap();
+    p.change_file(".cargo/config.toml", &output);
+
+    let git_dep_manifest = p.read_file("vendor/git_dep/Cargo.toml");
+    assert_e2e().eq(
+        git_dep_manifest,
+        str![[r##"
+# THIS FILE IS AUTOMATICALLY GENERATED BY CARGO
+#
+# When uploading crates to the registry Cargo will automatically
+# "normalize" Cargo.toml files for maximal compatibility
+# with all versions of Cargo and also rewrite `path` dependencies
+# to registry (e.g., crates.io) dependencies.
+#
+# If you are reading this file be aware that the original Cargo.toml
+# will likely look very different (and much more reasonable).
+# See Cargo.toml.orig for the original contents.
+
+[package]
+edition = "2021"
+name = "git_dep"
+version = "0.0.1"
+authors = []
+build = false
+autolib = false
+autobins = false
+autoexamples = false
+autotests = false
+autobenches = false
+description = "foo"
+documentation = "docs.rs/foo"
+readme = false
+license = "MIT"
+
+[lib]
+name = "git_dep"
+path = "src/lib.rs"
+
+[[example]]
+name = "a"
+path = "examples/a.rs"
+
+[[example]]
+name = "b"
+path = "examples/b.rs"
+
+[[example]]
+name = "c"
+path = "examples/c.rs"
+
+[[example]]
+name = "x"
+path = "examples/x.rs"
+
+[[example]]
+name = "y"
+path = "examples/y.rs"
+
+[[example]]
+name = "z"
+path = "examples/z.rs"
+
+"##]],
+    );
+}
+
+#[cargo_test]
+fn git_update_rev() {
+    let (git_project, git_repo) = git::new_repo("git", |p| {
+        p.file("Cargo.toml", &basic_manifest("a", "0.1.0"))
+            .file("src/lib.rs", "")
+    });
+    let url = git_project.url();
+    let ref_1 = "initial";
+    let ref_2 = "update";
+
+    git::tag(&git_repo, ref_1);
+
+    git_project.change_file("src/lib.rs", "pub fn f() {}");
+    git::add(&git_repo);
+    git::commit(&git_repo);
+    git::tag(&git_repo, ref_2);
+
+    let p = project()
+        .file(
+            "Cargo.toml",
+            &format!(
+                r#"
+                    [package]
+                    name = "foo"
+                    version = "0.1.0"
+
+                    [dependencies]
+                    a = {{ git = '{url}', rev = '{ref_1}' }}
+                "#
+            ),
+        )
+        .file("src/lib.rs", "")
+        .build();
+
+    p.cargo("vendor --respect-source-config --versioned-dirs")
+        .run();
+
+    let lib = p.read_file("vendor/a-0.1.0/src/lib.rs");
+    assert_e2e().eq(lib, "");
+
+    p.change_file(
+        "Cargo.toml",
+        &format!(
+            r#"
+                    [package]
+                    name = "foo"
+                    version = "0.1.0"
+
+                    [dependencies]
+                    a = {{ git = '{url}', rev = '{ref_2}' }}
+                "#
+        ),
+    );
+
+    p.cargo("vendor --respect-source-config --versioned-dirs")
+        .run();
+
+    let lib = p.read_file("vendor/a-0.1.0/src/lib.rs");
+    assert_e2e().eq(lib, "pub fn f() {}");
+}
+
 #[cargo_test]
 fn depend_on_vendor_dir_not_deleted() {
     let p = project()
@@ -958,6 +1722,7 @@ fn config_instructions_works() {
                 [package]
                 name = "foo"
                 version = "0.1.0"
+                edition = "2021"
 
                 [dependencies]
                 dep = "0.1"
@@ -969,17 +1734,26 @@ fn config_instructions_works() {
         )
         .file("src/lib.rs", "")
         .build();
-    let output = p
-        .cargo("vendor --respect-source-config")
-        .exec_with_output()
-        .unwrap();
+    let output = p.cargo("vendor --respect-source-config").run();
     let output = String::from_utf8(output.stdout).unwrap();
-    p.change_file(".cargo/config", &output);
+    p.change_file(".cargo/config.toml", &output);
 
     p.cargo("check -v")
-        .with_stderr_contains("[..]foo/vendor/dep/src/lib.rs[..]")
-        .with_stderr_contains("[..]foo/vendor/altdep/src/lib.rs[..]")
-        .with_stderr_contains("[..]foo/vendor/gitdep/src/lib.rs[..]")
+        .with_stderr_data(
+            str![[r#"
+[CHECKING] altdep v0.1.0 (registry `alternative`)
+[CHECKING] dep v0.1.0
+[RUNNING] `rustc [..] [ROOT]/foo/vendor/altdep/src/lib.rs [..]`
+[RUNNING] `rustc [..] [ROOT]/foo/vendor/gitdep/src/lib.rs [..]`
+[RUNNING] `rustc [..] [ROOT]/foo/vendor/dep/src/lib.rs [..]`
+[CHECKING] foo v0.1.0 ([ROOT]/foo)
+[RUNNING] `rustc [..] src/lib.rs [..]`
+[FINISHED] `dev` profile [unoptimized + debuginfo] target(s) in [ELAPSED]s
+[CHECKING] gitdep v0.5.0 ([ROOTURL]/gitdep#[..])
+
+"#]]
+            .unordered(),
+        )
         .run();
 }
 
@@ -1052,10 +1826,11 @@ fn vendor_preserves_permissions() {
 
     p.cargo("vendor --respect-source-config").run();
 
+    let umask = cargo::util::get_umask();
     let metadata = fs::metadata(p.root().join("vendor/bar/src/lib.rs")).unwrap();
-    assert_eq!(metadata.mode() & 0o777, 0o644);
+    assert_eq!(metadata.mode() & 0o777, 0o644 & !umask);
     let metadata = fs::metadata(p.root().join("vendor/bar/example.sh")).unwrap();
-    assert_eq!(metadata.mode() & 0o777, 0o755);
+    assert_eq!(metadata.mode() & 0o777, 0o755 & !umask);
 }
 
 #[cargo_test]
@@ -1084,7 +1859,11 @@ fn no_remote_dependency_no_vendor() {
         .build();
 
     p.cargo("vendor")
-        .with_stderr("There is no dependency to vendor in this project.")
+        .with_stderr_data(str![[r#"
+[LOCKING] 1 package to latest compatible version
+There is no dependency to vendor in this project.
+
+"#]])
         .run();
     assert!(!p.root().join("vendor").exists());
 }
@@ -1107,6 +1886,7 @@ fn vendor_crate_with_ws_inherit() {
                 [package]
                 name = "bar"
                 version.workspace = true
+                edition = "2021"
             "#,
         )
         .file("bar/src/lib.rs", "")
@@ -1120,6 +1900,7 @@ fn vendor_crate_with_ws_inherit() {
                     [package]
                     name = "foo"
                     version = "0.1.0"
+                    edition = "2021"
 
                     [dependencies]
                     bar = {{ git = '{}' }}
@@ -1132,7 +1913,7 @@ fn vendor_crate_with_ws_inherit() {
 
     p.cargo("vendor --respect-source-config").run();
     p.change_file(
-        ".cargo/config",
+        ".cargo/config.toml",
         &format!(
             r#"
                 [source."{}"]
@@ -1148,6 +1929,13 @@ fn vendor_crate_with_ws_inherit() {
     );
 
     p.cargo("check -v")
-        .with_stderr_contains("[..]foo/vendor/bar/src/lib.rs[..]")
+        .with_stderr_data(str![[r#"
+[CHECKING] bar v0.1.0 ([ROOTURL]/ws#[..])
+[RUNNING] `rustc [..] [ROOT]/foo/vendor/bar/src/lib.rs [..]`
+[CHECKING] foo v0.1.0 ([ROOT]/foo)
+[RUNNING] `rustc [..] src/lib.rs [..]`
+[FINISHED] `dev` profile [unoptimized + debuginfo] target(s) in [ELAPSED]s
+
+"#]])
         .run();
 }
